package notes

import (
	"fmt"
	"regexp"
	"time"

	"gorm.io/gorm"
)

// Note represents a note in the system
type Note struct {
	ID        string    `json:"id" gorm:"primaryKey"`
	Title     string    `json:"title" gorm:"not null"`
	Content   string    `json:"content" gorm:"not null"`
	CreatedAt time.Time `json:"createdAt"`
	UpdatedAt time.Time `json:"updatedAt"`
	// Link relationships
	LinksTo  []*Note `json:"linksTo,omitempty" gorm:"many2many:note_links;joinForeignKey:source_note_id;joinReferences:target_note_id"`
	LinkedBy []*Note `json:"linkedBy,omitempty" gorm:"many2many:note_links;joinForeignKey:target_note_id;joinReferences:source_note_id"`
}

// NoteLink represents a connection between two notes
type NoteLink struct {
	SourceNoteID string    `gorm:"primaryKey"`
	TargetNoteID string    `gorm:"primaryKey"`
	CreatedAt    time.Time `gorm:"autoCreateTime"`
}

// ExtractLinks finds all [[note-title]] style links in the content
func (n *Note) ExtractLinks(content string) []string {
	re := regexp.MustCompile(`\[\[(.*?)\]\]`)
	matches := re.FindAllStringSubmatch(content, -1)
	titles := make([]string, 0, len(matches))
	for _, match := range matches {
		if len(match) > 1 {
			titles = append(titles, match[1])
		}
	}
	return titles
}

// UpdateLinks updates the note's links based on its content
func (n *Note) UpdateLinks(db *gorm.DB) error {
	// Delete existing links
	if err := db.Where("source_note_id = ?", n.ID).Delete(&NoteLink{}).Error; err != nil {
		return fmt.Errorf("failed to delete existing links: %w", err)
	}

	// Extract and create new links
	titles := n.ExtractLinks(n.Content)
	for _, title := range titles {
		var target Note
		if err := db.Where("title = ?", title).First(&target).Error; err != nil {
			if err == gorm.ErrRecordNotFound {
				// Skip non-existent notes
				continue
			}
			return fmt.Errorf("failed to find target note %q: %w", title, err)
		}

		link := NoteLink{
			SourceNoteID: n.ID,
			TargetNoteID: target.ID,
		}
		if err := db.Create(&link).Error; err != nil {
			return fmt.Errorf("failed to create link to %q: %w", title, err)
		}
	}

	return nil
}
